<?php
namespace Teknisa\EST\Modules\EstoqueCego\Factories;

use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Est91000ProgramacaoDoInventarioCegoFactory {
    protected $connection;
    protected $entityManager;

    public function __construct ($entityManager) {
        $this->connection = $entityManager->getConnection();
		$this->entityManager = $entityManager;
    }
    
    public function insertLote($params) {
        $sql = "INSERT INTO LOTE
                (CDFILIAL, NRLOTEESTQ, NRSUBLOTE, CDPRODUTO, IDSTATLOTE, DTENTRLOTE)
                (SELECT :CDFILIAL CDFILIAL, :NRLOTEESTQ NRLOTEESTQ, :NRSUBLOTE NRSUBLOTE, :CDPRODUTO CDPRODUTO,
                'L' IDSTATLOTE, SYSDATE DTENTRLOTE
                FROM PARAMGERAL
                MINUS
                SELECT CDFILIAL, NRLOTEESTQ, NRSUBLOTE, :CDPRODUTO CDPRODUTO, 'L', SYSDATE
                FROM LOTE
                WHERE :CDFILIAL   =  CDFILIAL     AND
                      :NRLOTEESTQ =  NRLOTEESTQ   AND
                      :NRSUBLOTE  =  NRSUBLOTE    AND
                      :CDPRODUTO  =  CDPRODUTO)";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function deleteAllInventory($params) {
        $sql = "DELETE FROM INVENTESTCEGO
                 WHERE CDFILIAL = :CDFILIAL
                   AND DTCONTAGEM = :DTCONTAGEM";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function getCDFILISUBGR($params) {
        $sql = "SELECT DISTINCT CDFILISUBGR
                  FROM PRSUBGRPROD
                 WHERE CDGRUPPROD = :CDGRUPPROD
                 ORDER BY CDFILISUBGR ASC";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $result = $std->fetch();
        
        return $result;
    }
    
    public function getProductsInfo($params,$type) {
        switch($type){
            case "byGroupOnly":
                $sql = "SELECT DISTINCT CDPRODMOVI, CDALMOXARIFE,
                               CDLOCALESTOQ, NRLOTEESTQ, NRSUBLOTE
                          FROM ITLANCTOEST I, PRODUTO P, GRUPPROD GR,
                               SUBGRUPPROD SG, PRSUBGRPROD AN
                         WHERE CDFILIMOVI      = :CDFILIAL
                           AND NVL(IDEVOLESTOQ,'S') = 'S'
                           AND DTLANCMOVI BETWEEN :DTINICIAL AND :DTFINAL
                           AND CDARVPROD BETWEEN :CDPRODINI AND :CDPRODFIN
                           AND (P.CDPRODUTO    = CDPRODMOVI)
                           AND (CDFILISUBGR    = :CDFILISUBGR)
                           AND (GR.CDGRUPPROD  = :CDGRUPPROD)
                           AND (GR.CDGRUPPROD  = SG.CDGRUPPROD)
                           AND (SG.CDSUBGRPROD = AN.CDSUBGRPROD)
                           AND (I.CDPRODMOVI   = AN.CDPRODUTO)
                           AND (SG.CDGRUPPROD  = AN.CDGRUPPROD)
                         ORDER BY CDALMOXARIFE, CDLOCALESTOQ";
                break;
            case "byProductsOnly":
                $sql = "SELECT DISTINCT I.CDPRODMOVI, I.CDALMOXARIFE, I.CDLOCALESTOQ,
                               I.NRLOTEESTQ, I.NRSUBLOTE
                          FROM ITLANCTOEST I, PRODUTO P, PRODFILI F
                         WHERE CDFILIMOVI            = :CDFILIAL
                           AND NVL(IDEVOLESTOQ,'S')  = 'S'
                           AND (DTLANCMOVI BETWEEN :DTINICIAL AND :DTFINAL)
                           AND (CDARVPROD BETWEEN :CDPRODINI AND :CDPRODFIN)
                           AND (P.CDPRODUTO          = CDPRODMOVI)
                           AND (I.CDPRODMOVI         = F.CDPRODUTO)
                           AND (CDFILIMOVI           = F.CDFILIAL)
                           AND (NVL(IDCNTRESTOQ,'S') = 'S')
                         ORDER BY I.CDALMOXARIFE, I.CDLOCALESTOQ";
                break;
            case "byGroupWithParamChecked":
                $sql = "SELECT P.CDPRODUTO AS CDPRODMOVI, P.CDALMOXARIFE, P.CDLOCALESTOQ, 
                               P.NRLOTEESTQ, P.NRSUBLOTE
                          FROM POSESTDIA P, PRODFILI PF, GRUPPROD GR, SUBGRUPPROD SG, PRSUBGRPROD AN
                         WHERE P.CDFILIAL    = :CDFILIAL
                           AND P.CDFILIAL      = PF.CDFILIAL
                           AND P.CDPRODUTO     = PF.CDPRODUTO
                           AND (NVL(PF.IDCNTRESTOQ,'S') = 'S')
                           AND P.QTESTOQDIA    <> 0
                           AND P.CDPRODUTO BETWEEN :CDPRODINI AND :CDPRODFIN
                           AND (CDFILISUBGR    = :CDFILISUBGR)
                           AND (GR.CDGRUPPROD  = :CDGRUPPROD)
                           AND (GR.CDGRUPPROD  = SG.CDGRUPPROD)
                           AND (SG.CDSUBGRPROD = AN.CDSUBGRPROD)
                           AND (P.CDPRODUTO    = AN.CDPRODUTO)
                           AND (SG.CDGRUPPROD  = AN.CDGRUPPROD)
                           AND P.DTPOSIESTQ    =
                               (SELECT MAX(A.DTPOSIESTQ)
                                  FROM POSESTDIA A, PARAMFILIAL PA
                                 WHERE A.CDFILIAL   = P.CDFILIAL
                                   AND A.CDPRODUTO    = P.CDPRODUTO
                                   AND A.CDALMOXARIFE = P.CDALMOXARIFE
                                   AND A.CDLOCALESTOQ = P.CDLOCALESTOQ
                                   AND A.NRLOTEESTQ   = P.NRLOTEESTQ
                                   AND A.NRSUBLOTE    = P.NRSUBLOTE
                                   AND PA.CDFILIAL    = A.CDFILIAL
                                   AND A.DTPOSIESTQ  <= PA.DTPROCESSA )
                      ORDER BY P.CDALMOXARIFE, P.CDLOCALESTOQ, P.NRLOTEESTQ, P.NRSUBLOTE, P.CDPRODUTO";
                break;
            case "byProductsWithParamChecked":
                $sql = "SELECT P.CDPRODUTO AS CDPRODMOVI, P.CDALMOXARIFE, P.CDLOCALESTOQ,
                               P.NRLOTEESTQ, P.NRSUBLOTE
                          FROM POSESTDIA P, PRODFILI PF
                         WHERE P.CDFILIAL             = :CDFILIAL
                           AND P.CDFILIAL               = PF.CDFILIAL
                           AND P.CDPRODUTO              = PF.CDPRODUTO
                           AND (NVL(PF.IDCNTRESTOQ,'S') = 'S')
                           AND P.QTESTOQDIA            <> 0
                           AND P.CDPRODUTO BETWEEN :CDPRODINI AND :CDPRODFIN
                           AND P.DTPOSIESTQ =
                               (SELECT MAX(A.DTPOSIESTQ)
                                  FROM POSESTDIA A, PARAMFILIAL PA
                                 WHERE A.CDFILIAL   = P.CDFILIAL
                                   AND A.CDPRODUTO    = P.CDPRODUTO
                                   AND A.CDALMOXARIFE = P.CDALMOXARIFE
                                   AND A.CDLOCALESTOQ = P.CDLOCALESTOQ
                                   AND A.NRLOTEESTQ   = P.NRLOTEESTQ
                                   AND A.NRSUBLOTE    = P.NRSUBLOTE
                                   AND PA.CDFILIAL    = A.CDFILIAL
                                   AND A.DTPOSIESTQ  <= PA.DTPROCESSA )
                      ORDER BY P.CDALMOXARIFE, P.CDLOCALESTOQ, P.NRLOTEESTQ, P.NRSUBLOTE, P.CDPRODUTO";
                break;
        }
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $result = $std->fetchAll();
        return $result;
    }
}